@@ -40,7 +40,8 @@ import (
 )
 
 func init() {
-	// This can be removed when the default validation scheme in common is updated.
+	// This can be removed when the legacy global mode is fully deprecated.
+	//nolint:staticcheck
 	model.NameValidationScheme = model.UTF8Validation
 }
 
@@ -136,8 +137,8 @@ func TestCheckSDFile(t *testing.T) {
 		},
 		{
 			name: "bad file extension",
-			file: "./testdata/bad-sd-file-extension.nonexistant",
-			err:  "invalid file extension: \".nonexistant\"",
+			file: "./testdata/bad-sd-file-extension.nonexistent",
+			err:  "invalid file extension: \".nonexistent\"",
 		},
 		{
 			name: "bad format",
@@ -557,6 +558,16 @@ func TestCheckRules(t *testing.T) {
 	})
 }
 
+func TestCheckRulesWithFeatureFlag(t *testing.T) {
+	// As opposed to TestCheckRules calling CheckRules directly we run promtool
+	// so the feature flag parsing can be tested.
+
+	args := []string{"-test.main", "--enable-feature=promql-experimental-functions", "check", "rules", "testdata/features.yml"}
+	tool := exec.Command(promtoolPath, args...)
+	err := tool.Run()
+	require.NoError(t, err)
+}
+
 func TestCheckRulesWithRuleFiles(t *testing.T) {
 	t.Run("rules-good", func(t *testing.T) {
 		t.Parallel()
