@@ -19,15 +19,18 @@ import (
 	"io"
 	"os"
 	"path/filepath"
+	"runtime"
 	"sort"
 	"sync"
 	"testing"
 	"time"
 
+	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/model"
 	"github.com/stretchr/testify/require"
 	"go.uber.org/goleak"
 
+	"github.com/prometheus/prometheus/discovery"
 	"github.com/prometheus/prometheus/discovery/targetgroup"
 )
 
@@ -143,15 +146,28 @@ func (t *testRunner) run(files ...string) {
 	ctx, cancel := context.WithCancel(context.Background())
 	t.cancelSD = cancel
 	go func() {
-		NewDiscovery(
-			&SDConfig{
-				Files: files,
-				// Setting a high refresh interval to make sure that the tests only
-				// rely on file watches.
-				RefreshInterval: model.Duration(1 * time.Hour),
-			},
+		conf := &SDConfig{
+			Files: files,
+			// Setting a high refresh interval to make sure that the tests only
+			// rely on file watches.
+			RefreshInterval: model.Duration(1 * time.Hour),
+		}
+
+		reg := prometheus.NewRegistry()
+		refreshMetrics := discovery.NewRefreshMetrics(reg)
+		metrics := conf.NewDiscovererMetrics(reg, refreshMetrics)
+		require.NoError(t, metrics.Register())
+
+		d, err := NewDiscovery(
+			conf,
 			nil,
-		).Run(ctx, t.ch)
+			metrics,
+		)
+		require.NoError(t, err)
+
+		d.Run(ctx, t.ch)
+
+		metrics.Unregister()
 	}()
 }
 
@@ -188,11 +204,11 @@ func (t *testRunner) targets() []*targetgroup.Group {
 func (t *testRunner) requireUpdate(ref time.Time, expected []*targetgroup.Group) {
 	t.Helper()
 
+	timeout := time.After(defaultWait)
 	for {
 		select {
-		case <-time.After(defaultWait):
+		case <-timeout:
 			t.Fatalf("Expected update but got none")
-			return
 		case <-time.After(defaultWait / 10):
 			if ref.Equal(t.lastReceive()) {
 				// No update received.
@@ -304,10 +320,14 @@ func valid2Tg(file string) []*targetgroup.Group {
 }
 
 func TestInitialUpdate(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("flaky test, see https://github.com/prometheus/prometheus/issues/16212")
+	}
 	for _, tc := range []string{
 		"fixtures/valid.yml",
 		"fixtures/valid.json",
 	} {
+		tc := tc
 		t.Run(tc, func(t *testing.T) {
 			t.Parallel()
 
@@ -341,14 +361,15 @@ func TestInvalidFile(t *testing.T) {
 
 			// Verify that we've received nothing.
 			time.Sleep(defaultWait)
-			if runner.lastReceive().After(now) {
-				t.Fatalf("unexpected targets received: %v", runner.targets())
-			}
+			require.False(t, runner.lastReceive().After(now), "unexpected targets received: %v", runner.targets())
 		})
 	}
 }
 
 func TestNoopFileUpdate(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("flaky test, see https://github.com/prometheus/prometheus/issues/16212")
+	}
 	t.Parallel()
 
 	runner := newTestRunner(t)
@@ -367,6 +388,9 @@ func TestNoopFileUpdate(t *testing.T) {
 }
 
 func TestFileUpdate(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("flaky test, see https://github.com/prometheus/prometheus/issues/16212")
+	}
 	t.Parallel()
 
 	runner := newTestRunner(t)
@@ -385,6 +409,9 @@ func TestFileUpdate(t *testing.T) {
 }
 
 func TestInvalidFileUpdate(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("flaky test, see https://github.com/prometheus/prometheus/issues/16212")
+	}
 	t.Parallel()
 
 	runner := newTestRunner(t)
@@ -407,6 +434,9 @@ func TestInvalidFileUpdate(t *testing.T) {
 }
 
 func TestUpdateFileWithPartialWrites(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("flaky test, see https://github.com/prometheus/prometheus/issues/16212")
+	}
 	t.Parallel()
 
 	runner := newTestRunner(t)
