@@ -16,7 +16,6 @@ package kubernetes
 import (
 	"context"
 	"errors"
-	"fmt"
 	"log/slog"
 	"net"
 	"strconv"
@@ -106,13 +105,14 @@ func NewEndpointSlice(l *slog.Logger, eps cache.SharedIndexInformer, svc, pod, n
 		// LabelServiceName so this operation doesn't have to iterate over all
 		// endpoint objects.
 		for _, obj := range e.endpointSliceStore.List() {
-			esa, err := e.getEndpointSliceAdaptor(obj)
-			if err != nil {
+			es, ok := obj.(*v1.EndpointSlice)
+			if !ok {
 				e.logger.Error("converting to EndpointSlice object failed", "err", err)
 				continue
 			}
-			if lv, exists := esa.labels()[esa.labelServiceName()]; exists && lv == svc.Name {
-				e.enqueue(esa.get())
+			// Only consider the underlying EndpointSlices in the same namespace.
+			if svcName, exists := es.Labels[v1.LabelServiceName]; exists && svcName == svc.Name && es.Namespace == svc.Namespace {
+				e.enqueue(es)
 			}
 		}
 	}
@@ -229,27 +229,17 @@ func (e *EndpointSlice) process(ctx context.Context, ch chan<- []*targetgroup.Gr
 		return true
 	}
 
-	esa, err := e.getEndpointSliceAdaptor(o)
-	if err != nil {
-		e.logger.Error("converting to EndpointSlice object failed", "err", err)
-		return true
+	if es, ok := o.(*v1.EndpointSlice); ok {
+		send(ctx, ch, e.buildEndpointSlice(*es))
+	} else {
+		e.logger.Error("received unexpected object", "object", o)
+		return false
 	}
-
-	send(ctx, ch, e.buildEndpointSlice(esa))
 	return true
 }
 
-func (e *EndpointSlice) getEndpointSliceAdaptor(o interface{}) (endpointSliceAdaptor, error) {
-	switch endpointSlice := o.(type) {
-	case *v1.EndpointSlice:
-		return newEndpointSliceAdaptorFromV1(endpointSlice), nil
-	default:
-		return nil, fmt.Errorf("received unexpected object: %v", o)
-	}
-}
-
-func endpointSliceSource(ep endpointSliceAdaptor) string {
-	return endpointSliceSourceFromNamespaceAndName(ep.namespace(), ep.name())
+func endpointSliceSource(ep v1.EndpointSlice) string {
+	return endpointSliceSourceFromNamespaceAndName(ep.Namespace, ep.Name)
 }
 
 func endpointSliceSourceFromNamespaceAndName(namespace, name string) string {
@@ -274,95 +264,95 @@ const (
 	endpointSliceEndpointTopologyLabelPresentPrefix = metaLabelPrefix + "endpointslice_endpoint_topology_present_"
 )
 
-func (e *EndpointSlice) buildEndpointSlice(eps endpointSliceAdaptor) *targetgroup.Group {
+func (e *EndpointSlice) buildEndpointSlice(eps v1.EndpointSlice) *targetgroup.Group {
 	tg := &targetgroup.Group{
 		Source: endpointSliceSource(eps),
 	}
 	tg.Labels = model.LabelSet{
-		namespaceLabel:                lv(eps.namespace()),
-		endpointSliceAddressTypeLabel: lv(eps.addressType()),
+		namespaceLabel:                lv(eps.Namespace),
+		endpointSliceAddressTypeLabel: lv(string(eps.AddressType)),
 	}
 
-	addObjectMetaLabels(tg.Labels, eps.getObjectMeta(), RoleEndpointSlice)
+	addObjectMetaLabels(tg.Labels, eps.ObjectMeta, RoleEndpointSlice)
 
 	e.addServiceLabels(eps, tg)
 
 	type podEntry struct {
 		pod          *apiv1.Pod
-		servicePorts []endpointSlicePortAdaptor
+		servicePorts []v1.EndpointPort
 	}
 	seenPods := map[string]*podEntry{}
 
-	add := func(addr string, ep endpointSliceEndpointAdaptor, port endpointSlicePortAdaptor) {
+	add := func(addr string, ep v1.Endpoint, port v1.EndpointPort) {
 		a := addr
-		if port.port() != nil {
-			a = net.JoinHostPort(addr, strconv.FormatUint(uint64(*port.port()), 10))
+		if port.Port != nil {
+			a = net.JoinHostPort(addr, strconv.FormatUint(uint64(*port.Port), 10))
 		}
 
 		target := model.LabelSet{
 			model.AddressLabel: lv(a),
 		}
 
-		if port.name() != nil {
-			target[endpointSlicePortNameLabel] = lv(*port.name())
+		if port.Name != nil {
+			target[endpointSlicePortNameLabel] = lv(*port.Name)
 		}
 
-		if port.protocol() != nil {
-			target[endpointSlicePortProtocolLabel] = lv(*port.protocol())
+		if port.Protocol != nil {
+			target[endpointSlicePortProtocolLabel] = lv(string(*port.Protocol))
 		}
 
-		if port.port() != nil {
-			target[endpointSlicePortLabel] = lv(strconv.FormatUint(uint64(*port.port()), 10))
+		if port.Port != nil {
+			target[endpointSlicePortLabel] = lv(strconv.FormatUint(uint64(*port.Port), 10))
 		}
 
-		if port.appProtocol() != nil {
-			target[endpointSlicePortAppProtocol] = lv(*port.appProtocol())
+		if port.AppProtocol != nil {
+			target[endpointSlicePortAppProtocol] = lv(*port.AppProtocol)
 		}
 
-		if ep.conditions().ready() != nil {
-			target[endpointSliceEndpointConditionsReadyLabel] = lv(strconv.FormatBool(*ep.conditions().ready()))
+		if ep.Conditions.Ready != nil {
+			target[endpointSliceEndpointConditionsReadyLabel] = lv(strconv.FormatBool(*ep.Conditions.Ready))
 		}
 
-		if ep.conditions().serving() != nil {
-			target[endpointSliceEndpointConditionsServingLabel] = lv(strconv.FormatBool(*ep.conditions().serving()))
+		if ep.Conditions.Serving != nil {
+			target[endpointSliceEndpointConditionsServingLabel] = lv(strconv.FormatBool(*ep.Conditions.Serving))
 		}
 
-		if ep.conditions().terminating() != nil {
-			target[endpointSliceEndpointConditionsTerminatingLabel] = lv(strconv.FormatBool(*ep.conditions().terminating()))
+		if ep.Conditions.Terminating != nil {
+			target[endpointSliceEndpointConditionsTerminatingLabel] = lv(strconv.FormatBool(*ep.Conditions.Terminating))
 		}
 
-		if ep.hostname() != nil {
-			target[endpointSliceEndpointHostnameLabel] = lv(*ep.hostname())
+		if ep.Hostname != nil {
+			target[endpointSliceEndpointHostnameLabel] = lv(*ep.Hostname)
 		}
 
-		if ep.targetRef() != nil {
-			target[model.LabelName(endpointSliceAddressTargetKindLabel)] = lv(ep.targetRef().Kind)
-			target[model.LabelName(endpointSliceAddressTargetNameLabel)] = lv(ep.targetRef().Name)
+		if ep.TargetRef != nil {
+			target[model.LabelName(endpointSliceAddressTargetKindLabel)] = lv(ep.TargetRef.Kind)
+			target[model.LabelName(endpointSliceAddressTargetNameLabel)] = lv(ep.TargetRef.Name)
 		}
 
-		if ep.nodename() != nil {
-			target[endpointSliceEndpointNodenameLabel] = lv(*ep.nodename())
+		if ep.NodeName != nil {
+			target[endpointSliceEndpointNodenameLabel] = lv(*ep.NodeName)
 		}
 
-		if ep.zone() != nil {
-			target[model.LabelName(endpointSliceEndpointZoneLabel)] = lv(*ep.zone())
+		if ep.Zone != nil {
+			target[model.LabelName(endpointSliceEndpointZoneLabel)] = lv(*ep.Zone)
 		}
 
-		for k, v := range ep.topology() {
+		for k, v := range ep.DeprecatedTopology {
 			ln := strutil.SanitizeLabelName(k)
 			target[model.LabelName(endpointSliceEndpointTopologyLabelPrefix+ln)] = lv(v)
 			target[model.LabelName(endpointSliceEndpointTopologyLabelPresentPrefix+ln)] = presentValue
 		}
 
 		if e.withNodeMetadata {
-			if ep.targetRef() != nil && ep.targetRef().Kind == "Node" {
-				target = addNodeLabels(target, e.nodeInf, e.logger, &ep.targetRef().Name)
+			if ep.TargetRef != nil && ep.TargetRef.Kind == "Node" {
+				target = addNodeLabels(target, e.nodeInf, e.logger, &ep.TargetRef.Name)
 			} else {
-				target = addNodeLabels(target, e.nodeInf, e.logger, ep.nodename())
+				target = addNodeLabels(target, e.nodeInf, e.logger, ep.NodeName)
 			}
 		}
 
-		pod := e.resolvePodRef(ep.targetRef())
+		pod := e.resolvePodRef(ep.TargetRef)
 		if pod == nil {
 			// This target is not a Pod, so don't continue with Pod specific logic.
 			tg.Targets = append(tg.Targets, target)
@@ -383,12 +373,12 @@ func (e *EndpointSlice) buildEndpointSlice(eps endpointSliceAdaptor) *targetgrou
 		containers := append(pod.Spec.Containers, pod.Spec.InitContainers...)
 		for i, c := range containers {
 			for _, cport := range c.Ports {
-				if port.port() == nil {
+				if port.Port == nil {
 					continue
 				}
 
-				if *port.port() == cport.ContainerPort {
-					ports := strconv.FormatUint(uint64(*port.port()), 10)
+				if *port.Port == cport.ContainerPort {
+					ports := strconv.FormatUint(uint64(*port.Port), 10)
 					isInit := i >= len(pod.Spec.Containers)
 
 					target[podContainerNameLabel] = lv(c.Name)
@@ -408,9 +398,9 @@ func (e *EndpointSlice) buildEndpointSlice(eps endpointSliceAdaptor) *targetgrou
 		tg.Targets = append(tg.Targets, target)
 	}
 
-	for _, ep := range eps.endpoints() {
-		for _, port := range eps.ports() {
-			for _, addr := range ep.addresses() {
+	for _, ep := range eps.Endpoints {
+		for _, port := range eps.Ports {
+			for _, addr := range ep.Addresses {
 				add(addr, ep, port)
 			}
 		}
@@ -429,10 +419,10 @@ func (e *EndpointSlice) buildEndpointSlice(eps endpointSliceAdaptor) *targetgrou
 			for _, cport := range c.Ports {
 				hasSeenPort := func() bool {
 					for _, eport := range pe.servicePorts {
-						if eport.port() == nil {
+						if eport.Port == nil {
 							continue
 						}
-						if cport.ContainerPort == *eport.port() {
+						if cport.ContainerPort == *eport.Port {
 							return true
 						}
 					}
@@ -479,16 +469,16 @@ func (e *EndpointSlice) resolvePodRef(ref *apiv1.ObjectReference) *apiv1.Pod {
 	return obj.(*apiv1.Pod)
 }
 
-func (e *EndpointSlice) addServiceLabels(esa endpointSliceAdaptor, tg *targetgroup.Group) {
+func (e *EndpointSlice) addServiceLabels(esa v1.EndpointSlice, tg *targetgroup.Group) {
 	var (
 		found bool
 		name  string
 	)
-	ns := esa.namespace()
+	ns := esa.Namespace
 
 	// Every EndpointSlice object has the Service they belong to in the
 	// kubernetes.io/service-name label.
-	name, found = esa.labels()[esa.labelServiceName()]
+	name, found = esa.Labels[v1.LabelServiceName]
 	if !found {
 		return
 	}
