@@ -121,21 +121,18 @@ func (i *Ingress) process(ctx context.Context, ch chan<- []*targetgroup.Group) b
 		return true
 	}
 
-	var ia ingressAdaptor
-	switch ingress := o.(type) {
-	case *v1.Ingress:
-		ia = newIngressAdaptorFromV1(ingress)
-	default:
+	if ingress, ok := o.(*v1.Ingress); ok {
+		send(ctx, ch, i.buildIngress(*ingress))
+	} else {
 		i.logger.Error("converting to Ingress object failed", "err",
 			fmt.Errorf("received unexpected object: %v", o))
 		return true
 	}
-	send(ctx, ch, i.buildIngress(ia))
 	return true
 }
 
-func ingressSource(s ingressAdaptor) string {
-	return ingressSourceFromNamespaceAndName(s.namespace(), s.name())
+func ingressSource(s v1.Ingress) string {
+	return ingressSourceFromNamespaceAndName(s.Namespace, s.Name)
 }
 
 func ingressSourceFromNamespaceAndName(namespace, name string) string {
@@ -149,15 +146,15 @@ const (
 	ingressClassNameLabel = metaLabelPrefix + "ingress_class_name"
 )
 
-func ingressLabels(ingress ingressAdaptor) model.LabelSet {
+func ingressLabels(ingress v1.Ingress) model.LabelSet {
 	// Each label and annotation will create two key-value pairs in the map.
 	ls := make(model.LabelSet)
-	ls[namespaceLabel] = lv(ingress.namespace())
-	if cls := ingress.ingressClassName(); cls != nil {
+	ls[namespaceLabel] = lv(ingress.Namespace)
+	if cls := ingress.Spec.IngressClassName; cls != nil {
 		ls[ingressClassNameLabel] = lv(*cls)
 	}
 
-	addObjectMetaLabels(ls, ingress.getObjectMeta(), RoleIngress)
+	addObjectMetaLabels(ls, ingress.ObjectMeta, RoleIngress)
 
 	return ls
 }
@@ -177,19 +174,39 @@ func pathsFromIngressPaths(ingressPaths []string) []string {
 	return paths
 }
 
-func (i *Ingress) buildIngress(ingress ingressAdaptor) *targetgroup.Group {
+func rulePaths(rule v1.IngressRule) []string {
+	rv := rule.IngressRuleValue
+	if rv.HTTP == nil {
+		return nil
+	}
+	paths := make([]string, len(rv.HTTP.Paths))
+	for n, p := range rv.HTTP.Paths {
+		paths[n] = p.Path
+	}
+	return paths
+}
+
+func tlsHosts(ingressTLS []v1.IngressTLS) []string {
+	var hosts []string
+	for _, tls := range ingressTLS {
+		hosts = append(hosts, tls.Hosts...)
+	}
+	return hosts
+}
+
+func (i *Ingress) buildIngress(ingress v1.Ingress) *targetgroup.Group {
 	tg := &targetgroup.Group{
 		Source: ingressSource(ingress),
 	}
 	tg.Labels = ingressLabels(ingress)
 
-	for _, rule := range ingress.rules() {
+	for _, rule := range ingress.Spec.Rules {
 		scheme := "http"
-		paths := pathsFromIngressPaths(rule.paths())
+		paths := pathsFromIngressPaths(rulePaths(rule))
 
 	out:
-		for _, pattern := range ingress.tlsHosts() {
-			if matchesHostnamePattern(pattern, rule.host()) {
+		for _, pattern := range tlsHosts(ingress.Spec.TLS) {
+			if matchesHostnamePattern(pattern, rule.Host) {
 				scheme = "https"
 				break out
 			}
@@ -197,9 +214,9 @@ func (i *Ingress) buildIngress(ingress ingressAdaptor) *targetgroup.Group {
 
 		for _, path := range paths {
 			tg.Targets = append(tg.Targets, model.LabelSet{
-				model.AddressLabel: lv(rule.host()),
+				model.AddressLabel: lv(rule.Host),
 				ingressSchemeLabel: lv(scheme),
-				ingressHostLabel:   lv(rule.host()),
+				ingressHostLabel:   lv(rule.Host),
 				ingressPathLabel:   lv(path),
 			})
 		}
