@@ -25,13 +25,16 @@ import (
 	"github.com/prometheus/prometheus/discovery/targetgroup"
 )
 
-func makeNode(name, address string, labels, annotations map[string]string) *v1.Node {
+func makeNode(name, address, providerID string, labels, annotations map[string]string) *v1.Node {
 	return &v1.Node{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:        name,
 			Labels:      labels,
 			Annotations: annotations,
 		},
+		Spec: v1.NodeSpec{
+			ProviderID: providerID,
+		},
 		Status: v1.NodeStatus{
 			Addresses: []v1.NodeAddress{
 				{
@@ -49,10 +52,11 @@ func makeNode(name, address string, labels, annotations map[string]string) *v1.N
 }
 
 func makeEnumeratedNode(i int) *v1.Node {
-	return makeNode(fmt.Sprintf("test%d", i), "1.2.3.4", map[string]string{}, map[string]string{})
+	return makeNode(fmt.Sprintf("test%d", i), "1.2.3.4", fmt.Sprintf("aws:///de-west-3a/i-%d", i), map[string]string{}, map[string]string{})
 }
 
 func TestNodeDiscoveryBeforeStart(t *testing.T) {
+	t.Parallel()
 	n, c := makeDiscovery(RoleNode, NamespaceDiscovery{})
 
 	k8sDiscoveryTest{
@@ -61,6 +65,7 @@ func TestNodeDiscoveryBeforeStart(t *testing.T) {
 			obj := makeNode(
 				"test",
 				"1.2.3.4",
+				"aws:///nl-north-7b/i-03149834983492827",
 				map[string]string{"test-label": "testvalue"},
 				map[string]string{"test-annotation": "testannotationvalue"},
 			)
@@ -78,6 +83,7 @@ func TestNodeDiscoveryBeforeStart(t *testing.T) {
 				},
 				Labels: model.LabelSet{
 					"__meta_kubernetes_node_name":                              "test",
+					"__meta_kubernetes_node_provider_id":                       "aws:///nl-north-7b/i-03149834983492827",
 					"__meta_kubernetes_node_label_test_label":                  "testvalue",
 					"__meta_kubernetes_node_labelpresent_test_label":           "true",
 					"__meta_kubernetes_node_annotation_test_annotation":        "testannotationvalue",
@@ -90,6 +96,7 @@ func TestNodeDiscoveryBeforeStart(t *testing.T) {
 }
 
 func TestNodeDiscoveryAdd(t *testing.T) {
+	t.Parallel()
 	n, c := makeDiscovery(RoleNode, NamespaceDiscovery{})
 
 	k8sDiscoveryTest{
@@ -109,7 +116,8 @@ func TestNodeDiscoveryAdd(t *testing.T) {
 					},
 				},
 				Labels: model.LabelSet{
-					"__meta_kubernetes_node_name": "test1",
+					"__meta_kubernetes_node_name":        "test1",
+					"__meta_kubernetes_node_provider_id": "aws:///de-west-3a/i-1",
 				},
 				Source: "node/test1",
 			},
@@ -118,6 +126,7 @@ func TestNodeDiscoveryAdd(t *testing.T) {
 }
 
 func TestNodeDiscoveryDelete(t *testing.T) {
+	t.Parallel()
 	obj := makeEnumeratedNode(0)
 	n, c := makeDiscovery(RoleNode, NamespaceDiscovery{}, obj)
 
@@ -136,6 +145,7 @@ func TestNodeDiscoveryDelete(t *testing.T) {
 }
 
 func TestNodeDiscoveryUpdate(t *testing.T) {
+	t.Parallel()
 	n, c := makeDiscovery(RoleNode, NamespaceDiscovery{})
 
 	k8sDiscoveryTest{
@@ -146,6 +156,7 @@ func TestNodeDiscoveryUpdate(t *testing.T) {
 			obj2 := makeNode(
 				"test0",
 				"1.2.3.4",
+				"aws:///fr-south-1c/i-49508290343823952",
 				map[string]string{"Unschedulable": "true"},
 				map[string]string{},
 			)
@@ -165,6 +176,7 @@ func TestNodeDiscoveryUpdate(t *testing.T) {
 					"__meta_kubernetes_node_label_Unschedulable":        "true",
 					"__meta_kubernetes_node_labelpresent_Unschedulable": "true",
 					"__meta_kubernetes_node_name":                       "test0",
+					"__meta_kubernetes_node_provider_id":                "aws:///fr-south-1c/i-49508290343823952",
 				},
 				Source: "node/test0",
 			},
