@@ -15,43 +15,51 @@ package textparse
 
 import (
 	"bytes"
-	"encoding/binary"
 	"errors"
 	"fmt"
 	"io"
 	"math"
+	"strconv"
 	"strings"
+	"sync"
 	"unicode/utf8"
 
-	"github.com/gogo/protobuf/proto"
 	"github.com/gogo/protobuf/types"
 	"github.com/prometheus/common/model"
 
 	"github.com/prometheus/prometheus/model/exemplar"
 	"github.com/prometheus/prometheus/model/histogram"
 	"github.com/prometheus/prometheus/model/labels"
-
 	dto "github.com/prometheus/prometheus/prompb/io/prometheus/client"
 )
 
-// ProtobufParser is a very inefficient way of unmarshaling the old Prometheus
-// protobuf format and then present it as it if were parsed by a
-// Prometheus-2-style text parser. This is only done so that we can easily plug
-// in the protobuf format into Prometheus 2. For future use (with the final
-// format that will be used for native histograms), we have to revisit the
-// parsing. A lot of the efficiency tricks of the Prometheus-2-style parsing
-// could be used in a similar fashion (byte-slice pointers into the raw
-// payload), which requires some hand-coded protobuf handling. But the current
-// parsers all expect the full series name (metric name plus label pairs) as one
-// string, which is not how things are represented in the protobuf format. If
-// the re-arrangement work is actually causing problems (which has to be seen),
-// that expectation needs to be changed.
+// floatFormatBufPool is exclusively used in formatOpenMetricsFloat.
+var floatFormatBufPool = sync.Pool{
+	New: func() interface{} {
+		// To contain at most 17 digits and additional syntax for a float64.
+		b := make([]byte, 0, 24)
+		return &b
+	},
+}
+
+// ProtobufParser parses the old Prometheus protobuf format and present it
+// as the text-style textparse.Parser interface.
+//
+// It uses a tailored streaming protobuf dto.MetricStreamingDecoder that
+// reuses internal protobuf structs and allows direct unmarshalling to Prometheus
+// types like labels.
 type ProtobufParser struct {
-	in        []byte // The input to parse.
-	inPos     int    // Position within the input.
-	metricPos int    // Position within Metric slice.
+	dec *dto.MetricStreamingDecoder
+
+	// Used for both the string returned by Series and Histogram, as well as,
+	// metric family for Type, Unit and Help.
+	entryBytes *bytes.Buffer
+
+	lset    labels.Labels
+	builder labels.ScratchBuilder // Held here to reduce allocations when building Labels.
+
 	// fieldPos is the position within a Summary or (legacy) Histogram. -2
-	// is the count. -1 is the sum. Otherwise it is the index within
+	// is the count. -1 is the sum. Otherwise, it is the index within
 	// quantiles/buckets.
 	fieldPos    int
 	fieldsDone  bool // true if no more fields of a Summary or (legacy) Histogram to be processed.
@@ -67,27 +75,20 @@ type ProtobufParser struct {
 	// that we have to decode the next MetricFamily.
 	state Entry
 
-	builder labels.ScratchBuilder // held here to reduce allocations when building Labels
-
-	mf *dto.MetricFamily
-
 	// Whether to also parse a classic histogram that is also present as a
 	// native histogram.
 	parseClassicHistograms bool
-
-	// The following are just shenanigans to satisfy the Parser interface.
-	metricBytes *bytes.Buffer // A somewhat fluid representation of the current metric.
 }
 
 // NewProtobufParser returns a parser for the payload in the byte slice.
 func NewProtobufParser(b []byte, parseClassicHistograms bool, st *labels.SymbolTable) Parser {
 	return &ProtobufParser{
-		in:                     b,
+		dec:        dto.NewMetricStreamingDecoder(b),
+		entryBytes: &bytes.Buffer{},
+		builder:    labels.NewScratchBuilderWithSymbolTable(st, 16), // TODO(bwplotka): Try base builder.
+
 		state:                  EntryInvalid,
-		mf:                     &dto.MetricFamily{},
-		metricBytes:            &bytes.Buffer{},
 		parseClassicHistograms: parseClassicHistograms,
-		builder:                labels.NewScratchBuilderWithSymbolTable(st, 16),
 	}
 }
 
@@ -95,19 +96,18 @@ func NewProtobufParser(b []byte, parseClassicHistograms bool, st *labels.SymbolT
 // value, the timestamp if set, and the value of the current sample.
 func (p *ProtobufParser) Series() ([]byte, *int64, float64) {
 	var (
-		m  = p.mf.GetMetric()[p.metricPos]
-		ts = m.GetTimestampMs()
+		ts = &p.dec.TimestampMs // To save memory allocations, never nil.
 		v  float64
 	)
-	switch p.mf.GetType() {
+	switch p.dec.GetType() {
 	case dto.MetricType_COUNTER:
-		v = m.GetCounter().GetValue()
+		v = p.dec.GetCounter().GetValue()
 	case dto.MetricType_GAUGE:
-		v = m.GetGauge().GetValue()
+		v = p.dec.GetGauge().GetValue()
 	case dto.MetricType_UNTYPED:
-		v = m.GetUntyped().GetValue()
+		v = p.dec.GetUntyped().GetValue()
 	case dto.MetricType_SUMMARY:
-		s := m.GetSummary()
+		s := p.dec.GetSummary()
 		switch p.fieldPos {
 		case -2:
 			v = float64(s.GetSampleCount())
@@ -122,7 +122,7 @@ func (p *ProtobufParser) Series() ([]byte, *int64, float64) {
 		}
 	case dto.MetricType_HISTOGRAM, dto.MetricType_GAUGE_HISTOGRAM:
 		// This should only happen for a classic histogram.
-		h := m.GetHistogram()
+		h := p.dec.GetHistogram()
 		switch p.fieldPos {
 		case -2:
 			v = h.GetSampleCountFloat()
@@ -148,8 +148,8 @@ func (p *ProtobufParser) Series() ([]byte, *int64, float64) {
 	default:
 		panic("encountered unexpected metric type, this is a bug")
 	}
-	if ts != 0 {
-		return p.metricBytes.Bytes(), &ts, v
+	if *ts != 0 {
+		return p.entryBytes.Bytes(), ts, v
 	}
 	// TODO(beorn7): We assume here that ts==0 means no timestamp. That's
 	// not true in general, but proto3 originally has no distinction between
@@ -160,7 +160,7 @@ func (p *ProtobufParser) Series() ([]byte, *int64, float64) {
 	// away from gogo-protobuf to an actively maintained protobuf
 	// implementation. Once that's done, we can simply use the `optional`
 	// keyword and check for the unset state explicitly.
-	return p.metricBytes.Bytes(), nil, v
+	return p.entryBytes.Bytes(), nil, v
 }
 
 // Histogram returns the bytes of a series with a native histogram as a value,
@@ -175,47 +175,56 @@ func (p *ProtobufParser) Series() ([]byte, *int64, float64) {
 // value.
 func (p *ProtobufParser) Histogram() ([]byte, *int64, *histogram.Histogram, *histogram.FloatHistogram) {
 	var (
-		m  = p.mf.GetMetric()[p.metricPos]
-		ts = m.GetTimestampMs()
-		h  = m.GetHistogram()
+		ts = &p.dec.TimestampMs // To save memory allocations, never nil.
+		h  = p.dec.GetHistogram()
 	)
+
 	if p.parseClassicHistograms && len(h.GetBucket()) > 0 {
 		p.redoClassic = true
 	}
 	if h.GetSampleCountFloat() > 0 || h.GetZeroCountFloat() > 0 {
 		// It is a float histogram.
 		fh := histogram.FloatHistogram{
-			Count:           h.GetSampleCountFloat(),
-			Sum:             h.GetSampleSum(),
-			ZeroThreshold:   h.GetZeroThreshold(),
-			ZeroCount:       h.GetZeroCountFloat(),
-			Schema:          h.GetSchema(),
+			Count:         h.GetSampleCountFloat(),
+			Sum:           h.GetSampleSum(),
+			ZeroThreshold: h.GetZeroThreshold(),
+			ZeroCount:     h.GetZeroCountFloat(),
+			Schema:        h.GetSchema(),
+
+			// Decoder reuses slices, so we need to copy.
 			PositiveSpans:   make([]histogram.Span, len(h.GetPositiveSpan())),
-			PositiveBuckets: h.GetPositiveCount(),
+			PositiveBuckets: make([]float64, len(h.GetPositiveCount())),
 			NegativeSpans:   make([]histogram.Span, len(h.GetNegativeSpan())),
-			NegativeBuckets: h.GetNegativeCount(),
+			NegativeBuckets: make([]float64, len(h.GetNegativeCount())),
 		}
 		for i, span := range h.GetPositiveSpan() {
 			fh.PositiveSpans[i].Offset = span.GetOffset()
 			fh.PositiveSpans[i].Length = span.GetLength()
 		}
+		for i, cnt := range h.GetPositiveCount() {
+			fh.PositiveBuckets[i] = cnt
+		}
 		for i, span := range h.GetNegativeSpan() {
 			fh.NegativeSpans[i].Offset = span.GetOffset()
 			fh.NegativeSpans[i].Length = span.GetLength()
 		}
-		if p.mf.GetType() == dto.MetricType_GAUGE_HISTOGRAM {
+		for i, cnt := range h.GetNegativeCount() {
+			fh.NegativeBuckets[i] = cnt
+		}
+		if p.dec.GetType() == dto.MetricType_GAUGE_HISTOGRAM {
 			fh.CounterResetHint = histogram.GaugeType
 		}
 		fh.Compact(0)
-		if ts != 0 {
-			return p.metricBytes.Bytes(), &ts, nil, &fh
+		if *ts != 0 {
+			return p.entryBytes.Bytes(), ts, nil, &fh
 		}
 		// Nasty hack: Assume that ts==0 means no timestamp. That's not true in
 		// general, but proto3 has no distinction between unset and
 		// default. Need to avoid in the final format.
-		return p.metricBytes.Bytes(), nil, nil, &fh
+		return p.entryBytes.Bytes(), nil, nil, &fh
 	}
 
+	// TODO(bwplotka): Create sync.Pool for those structs.
 	sh := histogram.Histogram{
 		Count:           h.GetSampleCount(),
 		Sum:             h.GetSampleSum(),
@@ -223,41 +232,47 @@ func (p *ProtobufParser) Histogram() ([]byte, *int64, *histogram.Histogram, *his
 		ZeroCount:       h.GetZeroCount(),
 		Schema:          h.GetSchema(),
 		PositiveSpans:   make([]histogram.Span, len(h.GetPositiveSpan())),
-		PositiveBuckets: h.GetPositiveDelta(),
+		PositiveBuckets: make([]int64, len(h.GetPositiveDelta())),
 		NegativeSpans:   make([]histogram.Span, len(h.GetNegativeSpan())),
-		NegativeBuckets: h.GetNegativeDelta(),
+		NegativeBuckets: make([]int64, len(h.GetNegativeDelta())),
 	}
 	for i, span := range h.GetPositiveSpan() {
 		sh.PositiveSpans[i].Offset = span.GetOffset()
 		sh.PositiveSpans[i].Length = span.GetLength()
 	}
+	for i, cnt := range h.GetPositiveDelta() {
+		sh.PositiveBuckets[i] = cnt
+	}
 	for i, span := range h.GetNegativeSpan() {
 		sh.NegativeSpans[i].Offset = span.GetOffset()
 		sh.NegativeSpans[i].Length = span.GetLength()
 	}
-	if p.mf.GetType() == dto.MetricType_GAUGE_HISTOGRAM {
+	for i, cnt := range h.GetNegativeDelta() {
+		sh.NegativeBuckets[i] = cnt
+	}
+	if p.dec.GetType() == dto.MetricType_GAUGE_HISTOGRAM {
 		sh.CounterResetHint = histogram.GaugeType
 	}
 	sh.Compact(0)
-	if ts != 0 {
-		return p.metricBytes.Bytes(), &ts, &sh, nil
+	if *ts != 0 {
+		return p.entryBytes.Bytes(), ts, &sh, nil
 	}
-	return p.metricBytes.Bytes(), nil, &sh, nil
+	return p.entryBytes.Bytes(), nil, &sh, nil
 }
 
 // Help returns the metric name and help text in the current entry.
 // Must only be called after Next returned a help entry.
 // The returned byte slices become invalid after the next call to Next.
 func (p *ProtobufParser) Help() ([]byte, []byte) {
-	return p.metricBytes.Bytes(), []byte(p.mf.GetHelp())
+	return p.entryBytes.Bytes(), yoloBytes(p.dec.GetHelp())
 }
 
 // Type returns the metric name and type in the current entry.
 // Must only be called after Next returned a type entry.
 // The returned byte slices become invalid after the next call to Next.
 func (p *ProtobufParser) Type() ([]byte, model.MetricType) {
-	n := p.metricBytes.Bytes()
-	switch p.mf.GetType() {
+	n := p.entryBytes.Bytes()
+	switch p.dec.GetType() {
 	case dto.MetricType_COUNTER:
 		return n, model.MetricTypeCounter
 	case dto.MetricType_GAUGE:
@@ -276,7 +291,7 @@ func (p *ProtobufParser) Type() ([]byte, model.MetricType) {
 // Must only be called after Next returned a unit entry.
 // The returned byte slices become invalid after the next call to Next.
 func (p *ProtobufParser) Unit() ([]byte, []byte) {
-	return p.metricBytes.Bytes(), []byte(p.mf.GetUnit())
+	return p.entryBytes.Bytes(), []byte(p.dec.GetUnit())
 }
 
 // Comment always returns nil because comments aren't supported by the protobuf
@@ -285,24 +300,9 @@ func (p *ProtobufParser) Comment() []byte {
 	return nil
 }
 
-// Metric writes the labels of the current sample into the passed labels.
-// It returns the string from which the metric was parsed.
-func (p *ProtobufParser) Metric(l *labels.Labels) string {
-	p.builder.Reset()
-	p.builder.Add(labels.MetricName, p.getMagicName())
-
-	for _, lp := range p.mf.GetMetric()[p.metricPos].GetLabel() {
-		p.builder.Add(lp.GetName(), lp.GetValue())
-	}
-	if needed, name, value := p.getMagicLabel(); needed {
-		p.builder.Add(name, value)
-	}
-
-	// Sort labels to maintain the sorted labels invariant.
-	p.builder.Sort()
-	*l = p.builder.Labels()
-
-	return p.metricBytes.String()
+// Labels writes the labels of the current sample into the passed labels.
+func (p *ProtobufParser) Labels(l *labels.Labels) {
+	*l = p.lset.Copy()
 }
 
 // Exemplar writes the exemplar of the current sample into the passed
@@ -315,15 +315,14 @@ func (p *ProtobufParser) Exemplar(ex *exemplar.Exemplar) bool {
 		// We only ever return one exemplar per (non-native-histogram) series.
 		return false
 	}
-	m := p.mf.GetMetric()[p.metricPos]
 	var exProto *dto.Exemplar
-	switch p.mf.GetType() {
+	switch p.dec.GetType() {
 	case dto.MetricType_COUNTER:
-		exProto = m.GetCounter().GetExemplar()
+		exProto = p.dec.GetCounter().GetExemplar()
 	case dto.MetricType_HISTOGRAM, dto.MetricType_GAUGE_HISTOGRAM:
 		isClassic := p.state == EntrySeries
-		if !isClassic && len(m.GetHistogram().GetExemplars()) > 0 {
-			exs := m.GetHistogram().GetExemplars()
+		if !isClassic && len(p.dec.GetHistogram().GetExemplars()) > 0 {
+			exs := p.dec.GetHistogram().GetExemplars()
 			for p.exemplarPos < len(exs) {
 				exProto = exs[p.exemplarPos]
 				p.exemplarPos++
@@ -335,7 +334,7 @@ func (p *ProtobufParser) Exemplar(ex *exemplar.Exemplar) bool {
 				return false
 			}
 		} else {
-			bb := m.GetHistogram().GetBucket()
+			bb := p.dec.GetHistogram().GetBucket()
 			if p.fieldPos < 0 {
 				if isClassic {
 					return false // At _count or _sum.
@@ -379,26 +378,24 @@ func (p *ProtobufParser) Exemplar(ex *exemplar.Exemplar) bool {
 	return true
 }
 
-// CreatedTimestamp returns CT or nil if CT is not present or
-// invalid (as timestamp e.g. negative value) on counters, summaries or histograms.
-func (p *ProtobufParser) CreatedTimestamp() *int64 {
+// CreatedTimestamp returns CT or 0 if CT is not present on counters, summaries or histograms.
+func (p *ProtobufParser) CreatedTimestamp() int64 {
 	var ct *types.Timestamp
-	switch p.mf.GetType() {
+	switch p.dec.GetType() {
 	case dto.MetricType_COUNTER:
-		ct = p.mf.GetMetric()[p.metricPos].GetCounter().GetCreatedTimestamp()
+		ct = p.dec.GetCounter().GetCreatedTimestamp()
 	case dto.MetricType_SUMMARY:
-		ct = p.mf.GetMetric()[p.metricPos].GetSummary().GetCreatedTimestamp()
+		ct = p.dec.GetSummary().GetCreatedTimestamp()
 	case dto.MetricType_HISTOGRAM, dto.MetricType_GAUGE_HISTOGRAM:
-		ct = p.mf.GetMetric()[p.metricPos].GetHistogram().GetCreatedTimestamp()
+		ct = p.dec.GetHistogram().GetCreatedTimestamp()
 	default:
 	}
-	ctAsTime, err := types.TimestampFromProto(ct)
-	if err != nil {
-		// Errors means ct == nil or invalid timestamp, which we silently ignore.
-		return nil
+	if ct == nil {
+		return 0
 	}
-	ctMilis := ctAsTime.UnixMilli()
-	return &ctMilis
+	// Same as the gogo proto types.TimestampFromProto but straight to integer.
+	// and without validation.
+	return ct.GetSeconds()*1e3 + int64(ct.GetNanos())/1e6
 }
 
 // Next advances the parser to the next "sample" (emulating the behavior of a
@@ -407,31 +404,34 @@ func (p *ProtobufParser) CreatedTimestamp() *int64 {
 func (p *ProtobufParser) Next() (Entry, error) {
 	p.exemplarReturned = false
 	switch p.state {
+	// Invalid state occurs on:
+	// * First Next() call.
+	// * Recursive call that tells Next to move to the next metric family.
 	case EntryInvalid:
-		p.metricPos = 0
 		p.exemplarPos = 0
 		p.fieldPos = -2
-		n, err := readDelimited(p.in[p.inPos:], p.mf)
-		p.inPos += n
-		if err != nil {
+
+		if err := p.dec.NextMetricFamily(); err != nil {
 			return p.state, err
 		}
-
-		// Skip empty metric families.
-		if len(p.mf.GetMetric()) == 0 {
-			return p.Next()
+		if err := p.dec.NextMetric(); err != nil {
+			// Skip empty metric families.
+			if errors.Is(err, io.EOF) {
+				return p.Next()
+			}
+			return EntryInvalid, err
 		}
 
 		// We are at the beginning of a metric family. Put only the name
-		// into metricBytes and validate only name, help, and type for now.
-		name := p.mf.GetName()
+		// into entryBytes and validate only name, help, and type for now.
+		name := p.dec.GetName()
 		if !model.IsValidMetricName(model.LabelValue(name)) {
 			return EntryInvalid, fmt.Errorf("invalid metric name: %s", name)
 		}
-		if help := p.mf.GetHelp(); !utf8.ValidString(help) {
+		if help := p.dec.GetHelp(); !utf8.ValidString(help) {
 			return EntryInvalid, fmt.Errorf("invalid help for metric %q: %s", name, help)
 		}
-		switch p.mf.GetType() {
+		switch p.dec.GetType() {
 		case dto.MetricType_COUNTER,
 			dto.MetricType_GAUGE,
 			dto.MetricType_HISTOGRAM,
@@ -440,11 +440,11 @@ func (p *ProtobufParser) Next() (Entry, error) {
 			dto.MetricType_UNTYPED:
 			// All good.
 		default:
-			return EntryInvalid, fmt.Errorf("unknown metric type for metric %q: %s", name, p.mf.GetType())
+			return EntryInvalid, fmt.Errorf("unknown metric type for metric %q: %s", name, p.dec.GetType())
 		}
-		unit := p.mf.GetUnit()
+		unit := p.dec.GetUnit()
 		if len(unit) > 0 {
-			if p.mf.GetType() == dto.MetricType_COUNTER && strings.HasSuffix(name, "_total") {
+			if p.dec.GetType() == dto.MetricType_COUNTER && strings.HasSuffix(name, "_total") {
 				if !strings.HasSuffix(name[:len(name)-6], unit) || len(name)-6 < len(unit)+1 || name[len(name)-6-len(unit)-1] != '_' {
 					return EntryInvalid, fmt.Errorf("unit %q not a suffix of counter %q", unit, name)
 				}
@@ -452,55 +452,90 @@ func (p *ProtobufParser) Next() (Entry, error) {
 				return EntryInvalid, fmt.Errorf("unit %q not a suffix of metric %q", unit, name)
 			}
 		}
-		p.metricBytes.Reset()
-		p.metricBytes.WriteString(name)
-
+		p.entryBytes.Reset()
+		p.entryBytes.WriteString(name)
 		p.state = EntryHelp
 	case EntryHelp:
+		if p.dec.Unit != "" {
+			p.state = EntryUnit
+		} else {
+			p.state = EntryType
+		}
+	case EntryUnit:
 		p.state = EntryType
 	case EntryType:
-		t := p.mf.GetType()
+		t := p.dec.GetType()
 		if (t == dto.MetricType_HISTOGRAM || t == dto.MetricType_GAUGE_HISTOGRAM) &&
-			isNativeHistogram(p.mf.GetMetric()[0].GetHistogram()) {
+			isNativeHistogram(p.dec.GetHistogram()) {
 			p.state = EntryHistogram
 		} else {
 			p.state = EntrySeries
 		}
-		if err := p.updateMetricBytes(); err != nil {
+		if err := p.onSeriesOrHistogramUpdate(); err != nil {
 			return EntryInvalid, err
 		}
-	case EntryHistogram, EntrySeries:
-		if p.redoClassic {
-			p.redoClassic = false
-			p.state = EntrySeries
-			p.fieldPos = -3
-			p.fieldsDone = false
-		}
-		t := p.mf.GetType()
-		if p.state == EntrySeries && !p.fieldsDone &&
-			(t == dto.MetricType_SUMMARY ||
-				t == dto.MetricType_HISTOGRAM ||
-				t == dto.MetricType_GAUGE_HISTOGRAM) {
-			p.fieldPos++
-		} else {
-			p.metricPos++
+	case EntrySeries:
+		// Potentially a second series in the metric family.
+		t := p.dec.GetType()
+		if t == dto.MetricType_SUMMARY ||
+			t == dto.MetricType_HISTOGRAM ||
+			t == dto.MetricType_GAUGE_HISTOGRAM {
+			// Non-trivial series (complex metrics, with magic suffixes).
+
+			// Did we iterate over all the classic representations fields?
+			// NOTE: p.fieldsDone is updated on p.onSeriesOrHistogramUpdate.
+			if !p.fieldsDone {
+				// Still some fields to iterate over.
+				p.fieldPos++
+				if err := p.onSeriesOrHistogramUpdate(); err != nil {
+					return EntryInvalid, err
+				}
+				return p.state, nil
+			}
+
+			// Reset histogram fields.
 			p.fieldPos = -2
 			p.fieldsDone = false
 			p.exemplarPos = 0
+
 			// If this is a metric family containing native
-			// histograms, we have to switch back to native
-			// histograms after parsing a classic histogram.
-			if p.state == EntrySeries &&
-				(t == dto.MetricType_HISTOGRAM || t == dto.MetricType_GAUGE_HISTOGRAM) &&
-				isNativeHistogram(p.mf.GetMetric()[0].GetHistogram()) {
+			// histograms, it means we are here thanks to redoClassic state.
+			// Return to native histograms for the consistent flow.
+			if (t == dto.MetricType_HISTOGRAM || t == dto.MetricType_GAUGE_HISTOGRAM) &&
+				isNativeHistogram(p.dec.GetHistogram()) {
 				p.state = EntryHistogram
 			}
 		}
-		if p.metricPos >= len(p.mf.GetMetric()) {
-			p.state = EntryInvalid
-			return p.Next()
+		// Is there another series?
+		if err := p.dec.NextMetric(); err != nil {
+			if errors.Is(err, io.EOF) {
+				p.state = EntryInvalid
+				return p.Next()
+			}
+			return EntryInvalid, err
 		}
-		if err := p.updateMetricBytes(); err != nil {
+		if err := p.onSeriesOrHistogramUpdate(); err != nil {
+			return EntryInvalid, err
+		}
+	case EntryHistogram:
+		// Was Histogram() called and parseClassicHistograms is true?
+		if p.redoClassic {
+			p.redoClassic = false
+			p.fieldPos = -3
+			p.fieldsDone = false
+			p.state = EntrySeries
+			return p.Next() // Switch to classic histogram.
+		}
+
+		// Is there another series?
+		if err := p.dec.NextMetric(); err != nil {
+			if errors.Is(err, io.EOF) {
+				p.state = EntryInvalid
+				return p.Next()
+			}
+			return EntryInvalid, err
+		}
+		if err := p.onSeriesOrHistogramUpdate(); err != nil {
 			return EntryInvalid, err
 		}
 	default:
@@ -509,30 +544,39 @@ func (p *ProtobufParser) Next() (Entry, error) {
 	return p.state, nil
 }
 
-func (p *ProtobufParser) updateMetricBytes() error {
-	b := p.metricBytes
-	b.Reset()
-	b.WriteString(p.getMagicName())
-	for _, lp := range p.mf.GetMetric()[p.metricPos].GetLabel() {
-		b.WriteByte(model.SeparatorByte)
-		n := lp.GetName()
-		if !model.LabelName(n).IsValid() {
-			return fmt.Errorf("invalid label name: %s", n)
-		}
-		b.WriteString(n)
-		b.WriteByte(model.SeparatorByte)
-		v := lp.GetValue()
-		if !utf8.ValidString(v) {
-			return fmt.Errorf("invalid label value: %s", v)
-		}
-		b.WriteString(v)
+// onSeriesOrHistogramUpdate updates internal state before returning
+// a series or histogram. It updates:
+// * p.lset.
+// * p.entryBytes.
+// * p.fieldsDone depending on p.fieldPos.
+func (p *ProtobufParser) onSeriesOrHistogramUpdate() error {
+	p.builder.Reset()
+	p.builder.Add(labels.MetricName, p.getMagicName())
+
+	if err := p.dec.Label(&p.builder); err != nil {
+		return err
 	}
-	if needed, n, v := p.getMagicLabel(); needed {
-		b.WriteByte(model.SeparatorByte)
-		b.WriteString(n)
-		b.WriteByte(model.SeparatorByte)
-		b.WriteString(v)
+
+	if needed, name, value := p.getMagicLabel(); needed {
+		p.builder.Add(name, value)
 	}
+
+	// Sort labels to maintain the sorted labels invariant.
+	p.builder.Sort()
+	p.builder.Overwrite(&p.lset)
+
+	// entryBytes has to be unique for each series.
+	p.entryBytes.Reset()
+	p.lset.Range(func(l labels.Label) {
+		if l.Name == labels.MetricName {
+			p.entryBytes.WriteString(l.Value)
+			return
+		}
+		p.entryBytes.WriteByte(model.SeparatorByte)
+		p.entryBytes.WriteString(l.Name)
+		p.entryBytes.WriteByte(model.SeparatorByte)
+		p.entryBytes.WriteString(l.Value)
+	})
 	return nil
 }
 
@@ -540,36 +584,37 @@ func (p *ProtobufParser) updateMetricBytes() error {
 // ("_count", "_sum", "_bucket") if needed according to the current parser
 // state.
 func (p *ProtobufParser) getMagicName() string {
-	t := p.mf.GetType()
+	t := p.dec.GetType()
 	if p.state == EntryHistogram || (t != dto.MetricType_HISTOGRAM && t != dto.MetricType_GAUGE_HISTOGRAM && t != dto.MetricType_SUMMARY) {
-		return p.mf.GetName()
+		return p.dec.GetName()
 	}
 	if p.fieldPos == -2 {
-		return p.mf.GetName() + "_count"
+		return p.dec.GetName() + "_count"
 	}
 	if p.fieldPos == -1 {
-		return p.mf.GetName() + "_sum"
+		return p.dec.GetName() + "_sum"
 	}
 	if t == dto.MetricType_HISTOGRAM || t == dto.MetricType_GAUGE_HISTOGRAM {
-		return p.mf.GetName() + "_bucket"
+		return p.dec.GetName() + "_bucket"
 	}
-	return p.mf.GetName()
+	return p.dec.GetName()
 }
 
 // getMagicLabel returns if a magic label ("quantile" or "le") is needed and, if
 // so, its name and value. It also sets p.fieldsDone if applicable.
 func (p *ProtobufParser) getMagicLabel() (bool, string, string) {
+	// Native histogram or _count and _sum series.
 	if p.state == EntryHistogram || p.fieldPos < 0 {
 		return false, "", ""
 	}
-	switch p.mf.GetType() {
+	switch p.dec.GetType() {
 	case dto.MetricType_SUMMARY:
-		qq := p.mf.GetMetric()[p.metricPos].GetSummary().GetQuantile()
+		qq := p.dec.GetSummary().GetQuantile()
 		q := qq[p.fieldPos]
 		p.fieldsDone = p.fieldPos == len(qq)-1
 		return true, model.QuantileLabel, formatOpenMetricsFloat(q.GetQuantile())
 	case dto.MetricType_HISTOGRAM, dto.MetricType_GAUGE_HISTOGRAM:
-		bb := p.mf.GetMetric()[p.metricPos].GetHistogram().GetBucket()
+		bb := p.dec.GetHistogram().GetBucket()
 		if p.fieldPos >= len(bb) {
 			p.fieldsDone = true
 			return true, model.BucketLabel, "+Inf"
@@ -581,30 +626,7 @@ func (p *ProtobufParser) getMagicLabel() (bool, string, string) {
 	return false, "", ""
 }
 
-var errInvalidVarint = errors.New("protobufparse: invalid varint encountered")
-
-// readDelimited is essentially doing what the function of the same name in
-// github.com/matttproud/golang_protobuf_extensions/pbutil is doing, but it is
-// specific to a MetricFamily, utilizes the more efficient gogo-protobuf
-// unmarshaling, and acts on a byte slice directly without any additional
-// staging buffers.
-func readDelimited(b []byte, mf *dto.MetricFamily) (n int, err error) {
-	if len(b) == 0 {
-		return 0, io.EOF
-	}
-	messageLength, varIntLength := proto.DecodeVarint(b)
-	if varIntLength == 0 || varIntLength > binary.MaxVarintLen32 {
-		return 0, errInvalidVarint
-	}
-	totalLength := varIntLength + int(messageLength)
-	if totalLength > len(b) {
-		return 0, fmt.Errorf("protobufparse: insufficient length of buffer, expected at least %d bytes, got %d bytes", totalLength, len(b))
-	}
-	mf.Reset()
-	return totalLength, mf.Unmarshal(b[varIntLength:totalLength])
-}
-
-// formatOpenMetricsFloat works like the usual Go string formatting of a fleat
+// formatOpenMetricsFloat works like the usual Go string formatting of a float
 // but appends ".0" if the resulting number would otherwise contain neither a
 // "." nor an "e".
 func formatOpenMetricsFloat(f float64) string {
@@ -623,11 +645,15 @@ func formatOpenMetricsFloat(f float64) string {
 	case math.IsInf(f, -1):
 		return "-Inf"
 	}
-	s := fmt.Sprint(f)
-	if strings.ContainsAny(s, "e.") {
-		return s
+	bp := floatFormatBufPool.Get().(*[]byte)
+	defer floatFormatBufPool.Put(bp)
+
+	*bp = strconv.AppendFloat((*bp)[:0], f, 'g', -1, 64)
+	if bytes.ContainsAny(*bp, "e.") {
+		return string(*bp)
 	}
-	return s + ".0"
+	*bp = append(*bp, '.', '0')
+	return string(*bp)
 }
 
 // isNativeHistogram returns false iff the provided histograms has no spans at
