@@ -18,10 +18,12 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"maps"
 	"net/http"
 	"net/http/httptest"
 	"net/url"
 	"os"
+	"slices"
 	"sort"
 	"strconv"
 	"sync"
@@ -38,10 +40,6 @@ import (
 	"google.golang.org/protobuf/types/known/timestamppb"
 	"gopkg.in/yaml.v2"
 
-	"github.com/prometheus/prometheus/storage"
-
-	"github.com/prometheus/prometheus/model/timestamp"
-
 	"github.com/prometheus/prometheus/config"
 	"github.com/prometheus/prometheus/discovery"
 	_ "github.com/prometheus/prometheus/discovery/file"
@@ -49,19 +47,16 @@ import (
 	"github.com/prometheus/prometheus/model/histogram"
 	"github.com/prometheus/prometheus/model/labels"
 	"github.com/prometheus/prometheus/model/relabel"
+	"github.com/prometheus/prometheus/model/timestamp"
+	"github.com/prometheus/prometheus/storage"
 	"github.com/prometheus/prometheus/tsdb/tsdbutil"
 	"github.com/prometheus/prometheus/util/runutil"
 	"github.com/prometheus/prometheus/util/testutil"
 )
 
-func init() {
-	// This can be removed when the default validation scheme in common is updated.
-	model.NameValidationScheme = model.UTF8Validation
-}
-
 func TestPopulateLabels(t *testing.T) {
 	cases := []struct {
-		in      labels.Labels
+		in      model.LabelSet
 		cfg     *config.ScrapeConfig
 		res     labels.Labels
 		resOrig labels.Labels
@@ -69,10 +64,10 @@ func TestPopulateLabels(t *testing.T) {
 	}{
 		// Regular population of scrape config options.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel: "1.2.3.4:1000",
 				"custom":           "value",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -103,14 +98,14 @@ func TestPopulateLabels(t *testing.T) {
 		// Pre-define/overwrite scrape config labels.
 		// Leave out port and expect it to be defaulted to scheme.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel:        "1.2.3.4",
 				model.SchemeLabel:         "http",
 				model.MetricsPathLabel:    "/custom",
 				model.JobLabel:            "custom-job",
 				model.ScrapeIntervalLabel: "2s",
 				model.ScrapeTimeoutLabel:  "2s",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -138,10 +133,10 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Provide instance label. HTTPS port default for IPv6.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel:  "[::1]",
 				model.InstanceLabel: "custom-instance",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -170,7 +165,7 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Address label missing.
 		{
-			in: labels.FromStrings("custom", "value"),
+			in: model.LabelSet{"custom": "value"},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -184,7 +179,7 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Address label missing, but added in relabelling.
 		{
-			in: labels.FromStrings("custom", "host:1234"),
+			in: model.LabelSet{"custom": "host:1234"},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -222,7 +217,7 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Address label missing, but added in relabelling.
 		{
-			in: labels.FromStrings("custom", "host:1234"),
+			in: model.LabelSet{"custom": "host:1234"},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -260,10 +255,10 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Invalid UTF-8 in label.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel: "1.2.3.4:1000",
 				"custom":           "\xbd",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -277,10 +272,10 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Invalid duration in interval label.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel:        "1.2.3.4:1000",
 				model.ScrapeIntervalLabel: "2notseconds",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -294,10 +289,10 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Invalid duration in timeout label.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel:       "1.2.3.4:1000",
 				model.ScrapeTimeoutLabel: "2notseconds",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -311,10 +306,10 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// 0 interval in timeout label.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel:        "1.2.3.4:1000",
 				model.ScrapeIntervalLabel: "0s",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -328,10 +323,10 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// 0 duration in timeout label.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel:       "1.2.3.4:1000",
 				model.ScrapeTimeoutLabel: "0s",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -345,11 +340,11 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Timeout less than interval.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel:        "1.2.3.4:1000",
 				model.ScrapeIntervalLabel: "1s",
 				model.ScrapeTimeoutLabel:  "2s",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -363,9 +358,9 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// Don't attach default port.
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel: "1.2.3.4",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -393,9 +388,9 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// verify that the default port is not removed (http).
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel: "1.2.3.4:80",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "http",
 				MetricsPath:    "/metrics",
@@ -423,9 +418,9 @@ func TestPopulateLabels(t *testing.T) {
 		},
 		// verify that the default port is not removed (https).
 		{
-			in: labels.FromMap(map[string]string{
+			in: model.LabelSet{
 				model.AddressLabel: "1.2.3.4:443",
-			}),
+			},
 			cfg: &config.ScrapeConfig{
 				Scheme:         "https",
 				MetricsPath:    "/metrics",
@@ -453,17 +448,18 @@ func TestPopulateLabels(t *testing.T) {
 		},
 	}
 	for _, c := range cases {
-		in := c.in.Copy()
-
-		res, orig, err := PopulateLabels(labels.NewBuilder(c.in), c.cfg)
+		in := maps.Clone(c.in)
+		lb := labels.NewBuilder(labels.EmptyLabels())
+		res, err := PopulateLabels(lb, c.cfg, c.in, nil)
 		if c.err != "" {
 			require.EqualError(t, err, c.err)
 		} else {
 			require.NoError(t, err)
+			testutil.RequireEqual(t, c.res, res)
+			PopulateDiscoveredLabels(lb, c.cfg, c.in, nil)
+			testutil.RequireEqual(t, c.resOrig, lb.Labels())
 		}
-		require.Equal(t, c.in, in)
-		testutil.RequireEqual(t, c.res, res)
-		testutil.RequireEqual(t, c.resOrig, orig)
+		require.Equal(t, c.in, in) // Check this wasn't altered by PopulateLabels().
 	}
 }
 
@@ -477,7 +473,7 @@ func loadConfiguration(t testing.TB, c string) *config.Config {
 
 func noopLoop() loop {
 	return &testLoop{
-		startFunc: func(interval, timeout time.Duration, errc chan<- error) {},
+		startFunc: func(_, _ time.Duration, _ chan<- error) {},
 		stopFunc:  func() {},
 	}
 }
@@ -728,7 +724,7 @@ func setupTestServer(t *testing.T, typ string, toWrite []byte) *httptest.Server
 	once := sync.Once{}
 
 	server := httptest.NewServer(
-		http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 			fail := true
 			once.Do(func() {
 				fail = false
@@ -970,7 +966,7 @@ func TestManagerCTZeroIngestionHistogram(t *testing.T) {
 			once := sync.Once{}
 			// Start fake HTTP target to that allow one scrape only.
 			server := httptest.NewServer(
-				http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+				http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 					fail := true
 					once.Do(func() {
 						fail = false
@@ -1154,12 +1150,7 @@ func requireTargets(
 		}
 		sort.Strings(expectedTargets)
 		sort.Strings(sTargets)
-		for i, t := range sTargets {
-			if t != expectedTargets[i] {
-				return false
-			}
-		}
-		return true
+		return slices.Equal(sTargets, expectedTargets)
 	}, 1*time.Second, 100*time.Millisecond)
 }
 
