@@ -268,7 +268,7 @@ func TestHistogramChunkBucketChanges(t *testing.T) {
 	require.Empty(t, backwardPositiveInserts)
 	require.Empty(t, backwardNegativeInserts)
 	require.True(t, ok) // Only new buckets came in.
-	require.False(t, cr)
+	require.Equal(t, NotCounterReset, cr)
 	c, app = hApp.recode(posInterjections, negInterjections, h2.PositiveSpans, h2.NegativeSpans)
 	chk, _, _, err = app.AppendHistogram(nil, ts2, h2, false)
 	require.NoError(t, err)
@@ -394,7 +394,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.Empty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.True(t, ok) // Only new buckets came in.
-		require.False(t, cr)
+		require.Equal(t, NotCounterReset, cr)
 
 		assertRecodedHistogramChunkOnAppend(t, c, hApp, ts+1, h2, UnknownCounterReset)
 	}
@@ -417,7 +417,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.Empty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.False(t, ok) // Need to cut a new chunk.
-		require.True(t, cr)
+		require.Equal(t, CounterReset, cr)
 
 		assertNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, CounterReset, histogram.UnknownCounterReset)
 	}
@@ -443,7 +443,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.NotEmpty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.True(t, ok)
-		require.False(t, cr)
+		require.Equal(t, NotCounterReset, cr)
 
 		assertNoNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, UnknownCounterReset)
 
@@ -474,7 +474,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.NotEmpty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.True(t, ok)
-		require.False(t, cr)
+		require.Equal(t, NotCounterReset, cr)
 
 		assertRecodedHistogramChunkOnAppend(t, c, hApp, ts+1, h2, UnknownCounterReset)
 
@@ -502,7 +502,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.Empty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.False(t, ok) // Need to cut a new chunk.
-		require.True(t, cr)
+		require.Equal(t, CounterReset, cr)
 
 		assertNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, CounterReset, histogram.UnknownCounterReset)
 	}
@@ -528,7 +528,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.Empty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.False(t, ok) // Need to cut a new chunk.
-		require.True(t, cr)
+		require.Equal(t, CounterReset, cr)
 
 		assertNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, CounterReset, histogram.UnknownCounterReset)
 	}
@@ -560,7 +560,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.Empty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.False(t, ok) // Need to cut a new chunk.
-		require.True(t, cr)
+		require.Equal(t, CounterReset, cr)
 
 		assertNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, CounterReset, histogram.UnknownCounterReset)
 	}
@@ -667,7 +667,7 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.NotEmpty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.True(t, ok)
-		require.False(t, cr)
+		require.Equal(t, NotCounterReset, cr)
 
 		assertNoNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, UnknownCounterReset)
 	}
@@ -735,10 +735,42 @@ func TestHistogramChunkAppendable(t *testing.T) {
 		require.Empty(t, backwardPositiveInserts)
 		require.Empty(t, backwardNegativeInserts)
 		require.True(t, ok) // Only new buckets came in.
-		require.False(t, cr)
+		require.Equal(t, NotCounterReset, cr)
 
 		assertRecodedHistogramChunkOnAppend(t, c, hApp, ts+1, h2, UnknownCounterReset)
 	}
+
+	{ // New histogram with a different schema.
+		c, hApp, ts, h1 := setup(eh)
+		h2 := h1.Copy()
+		h2.Schema = 2
+
+		posInterjections, negInterjections, backwardPositiveInserts, backwardNegativeInserts, ok, cr := hApp.appendable(h2)
+		require.Empty(t, posInterjections)
+		require.Empty(t, negInterjections)
+		require.Empty(t, backwardPositiveInserts)
+		require.Empty(t, backwardNegativeInserts)
+		require.False(t, ok) // Need to cut a new chunk.
+		require.Equal(t, UnknownCounterReset, cr)
+
+		assertNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, UnknownCounterReset, histogram.UnknownCounterReset)
+	}
+
+	{ // New histogram with a different schema.
+		c, hApp, ts, h1 := setup(eh)
+		h2 := h1.Copy()
+		h2.ZeroThreshold = 1e-120
+
+		posInterjections, negInterjections, backwardPositiveInserts, backwardNegativeInserts, ok, cr := hApp.appendable(h2)
+		require.Empty(t, posInterjections)
+		require.Empty(t, negInterjections)
+		require.Empty(t, backwardPositiveInserts)
+		require.Empty(t, backwardNegativeInserts)
+		require.False(t, ok) // Need to cut a new chunk.
+		require.Equal(t, UnknownCounterReset, cr)
+
+		assertNewHistogramChunkOnAppend(t, c, hApp, ts+1, h2, UnknownCounterReset, histogram.UnknownCounterReset)
+	}
 }
 
 func assertNewHistogramChunkOnAppend(t *testing.T, oldChunk Chunk, hApp *HistogramAppender, ts int64, h *histogram.Histogram, expectHeader CounterResetHeader, expectHint histogram.CounterResetHint) {
@@ -1007,7 +1039,7 @@ func TestHistogramChunkAppendableWithEmptySpan(t *testing.T) {
 			require.Empty(t, bpI)
 			require.Empty(t, bnI)
 			require.True(t, okToAppend)
-			require.False(t, counterReset)
+			require.Equal(t, NotCounterReset, counterReset)
 		})
 	}
 }
