@@ -14,6 +14,7 @@
 package tsdb
 
 import (
+	"context"
 	"os"
 	"path/filepath"
 	"testing"
@@ -24,10 +25,12 @@ import (
 	"github.com/prometheus/prometheus/tsdb/chunks"
 	"github.com/prometheus/prometheus/tsdb/fileutil"
 	"github.com/prometheus/prometheus/tsdb/index"
+	"github.com/prometheus/prometheus/util/testutil"
 )
 
 func TestRepairBadIndexVersion(t *testing.T) {
 	tmpDir := t.TempDir()
+	ctx := context.Background()
 
 	// The broken index used in this test was written by the following script
 	// at a broken revision.
@@ -76,15 +79,15 @@ func TestRepairBadIndexVersion(t *testing.T) {
 	require.NoError(t, os.MkdirAll(filepath.Join(tmpDbDir, "chunks"), 0o777))
 
 	// Read current index to check integrity.
-	r, err := index.NewFileReader(filepath.Join(tmpDbDir, indexFilename))
+	r, err := index.NewFileReader(filepath.Join(tmpDbDir, indexFilename), index.DecodePostingsRaw)
 	require.NoError(t, err)
-	p, err := r.Postings("b", "1")
+	p, err := r.Postings(ctx, "b", "1")
 	require.NoError(t, err)
+	var builder labels.ScratchBuilder
 	for p.Next() {
 		t.Logf("next ID %d", p.At())
 
-		var lset labels.Labels
-		require.Error(t, r.Series(p.At(), &lset, nil))
+		require.Error(t, r.Series(p.At(), &builder, nil))
 	}
 	require.NoError(t, p.Err())
 	require.NoError(t, r.Close())
@@ -94,24 +97,23 @@ func TestRepairBadIndexVersion(t *testing.T) {
 	require.NoError(t, err)
 	db.Close()
 
-	r, err = index.NewFileReader(filepath.Join(tmpDbDir, indexFilename))
+	r, err = index.NewFileReader(filepath.Join(tmpDbDir, indexFilename), index.DecodePostingsRaw)
 	require.NoError(t, err)
 	defer r.Close()
-	p, err = r.Postings("b", "1")
+	p, err = r.Postings(ctx, "b", "1")
 	require.NoError(t, err)
 	res := []labels.Labels{}
 
 	for p.Next() {
 		t.Logf("next ID %d", p.At())
 
-		var lset labels.Labels
 		var chks []chunks.Meta
-		require.NoError(t, r.Series(p.At(), &lset, &chks))
-		res = append(res, lset)
+		require.NoError(t, r.Series(p.At(), &builder, &chks))
+		res = append(res, builder.Labels())
 	}
 
 	require.NoError(t, p.Err())
-	require.Equal(t, []labels.Labels{
+	testutil.RequireEqual(t, []labels.Labels{
 		labels.FromStrings("a", "1", "b", "1"),
 		labels.FromStrings("a", "2", "b", "1"),
 	}, res)
