@@ -12,6 +12,7 @@ import {
   TextInput,
   Anchor,
   Pagination,
+  rem,
 } from "@mantine/core";
 import { useSuspenseAPIQuery } from "../api/api";
 import { AlertingRule, AlertingRulesResult } from "../api/responseTypes/rules";
@@ -23,10 +24,10 @@ import { Fragment, useEffect, useMemo } from "react";
 import { StateMultiSelect } from "../components/StateMultiSelect";
 import { IconInfoCircle, IconSearch } from "@tabler/icons-react";
 import { LabelBadges } from "../components/LabelBadges";
+import { useLocalStorage } from "@mantine/hooks";
 import { useSettings } from "../state/settingsSlice";
 import {
   ArrayParam,
-  BooleanParam,
   NumberParam,
   StringParam,
   useQueryParam,
@@ -162,10 +163,10 @@ export default function AlertsPage() {
     withDefault(StringParam, "")
   );
   const [debouncedSearch] = useDebouncedValue<string>(searchFilter.trim(), 250);
-  const [showEmptyGroups, setShowEmptyGroups] = useQueryParam(
-    "showEmptyGroups",
-    withDefault(BooleanParam, true)
-  );
+  const [showEmptyGroups, setShowEmptyGroups] = useLocalStorage<boolean>({
+    key: "alertsPage.showEmptyGroups",
+    defaultValue: false,
+  });
 
   const { alertGroupsPerPage } = useSettings();
   const [activePage, setActivePage] = useQueryParam(
@@ -215,14 +216,14 @@ export default function AlertsPage() {
   // convenient to have in the same file IMO).
   const renderedPageItems = useMemo(
     () =>
-      currentPageGroups.map((g, i) => (
+      currentPageGroups.map((g) => (
         <Card
           shadow="xs"
           withBorder
           p="md"
-          key={i} // TODO: Find a stable and definitely unique key.
+          key={`${g.file}-${g.name}`}
         >
-          <Group mb="md" mt="xs" ml="xs" justify="space-between">
+          <Group mb="sm" justify="space-between">
             <Group align="baseline">
               <Text fz="xl" fw={600} c="var(--mantine-primary-color-filled)">
                 {g.name}
@@ -280,6 +281,7 @@ export default function AlertsPage() {
                   {items.map((r, j) => {
                     return (
                       <Accordion.Item
+                        mt={rem(5)}
                         styles={{
                           item: {
                             // TODO: This transparency hack is an OK workaround to make the collapsed items
@@ -299,7 +301,9 @@ export default function AlertsPage() {
                               : panelClasses.panelHealthOk
                         }
                       >
-                        <Accordion.Control>
+                        <Accordion.Control
+                          styles={{ label: { paddingBlock: rem(10) } }}
+                        >
                           <Group wrap="nowrap" justify="space-between" mr="lg">
                             <Text>{r.rule.name}</Text>
                             <Group gap="xs">
@@ -423,7 +427,7 @@ export default function AlertsPage() {
               o as keyof typeof alertsPageData.globalCounts
             ]
           }
-          placeholder="Filter by rule state"
+          placeholder="Filter by rule group state"
           values={(stateFilter?.filter((v) => v !== null) as string[]) || []}
           onChange={(values) => setStateFilter(values)}
         />
