@@ -13,8 +13,7 @@
 
 import { buildVectorMatching } from './vector';
 import { createEditorState } from '../test/utils-test';
-import { walkThrough } from './path-finder';
-import { BinaryExpr, Expr } from '@prometheus-io/lezer-promql';
+import { BinaryExpr } from '@prometheus-io/lezer-promql';
 import { syntaxTree } from '@codemirror/language';
 import { VectorMatchCardinality } from '../types';
 
@@ -201,7 +200,7 @@ describe('buildVectorMatching test', () => {
   testCases.forEach((value) => {
     it(value.binaryExpr, () => {
       const state = createEditorState(value.binaryExpr);
-      const node = walkThrough(syntaxTree(state).topNode, Expr, BinaryExpr);
+      const node = syntaxTree(state).topNode.getChild(BinaryExpr);
       expect(node).toBeTruthy();
       if (node) {
         expect(value.expectedVectorMatching).toEqual(buildVectorMatching(state, node));
